export interface PointLike {
  x: number
  y: number
}
export interface BoundingBox {
  min: PointLike
  max: PointLike
}
export interface Geometry {
  pos: PointLike
  aabb: BoundingBox
}
export default class SpatialManager<T> {
  SceneWidth: number
  SceneHeight: number
  cellSize: number
  private cellSizeInv
  private numCols
  private numRows
  numBuckets: number
  buckets: Map<number, Set<T>>
  constructor(SceneWidth: number, SceneHeight: number, cellSize: number)
  /**
   * Empties Spatial Hashmap and reinitializes.
   */
  static clearMap<U>(buckets: Map<number, Set<U>>, numBuckets: number): void
  /**
   * Empties Spatial Hashmap and reinitializes.
   */
  clearMap(): void
  /**
   * How you get your obj into the map
   * @param obj - what you want to register in the spatial hashmap
   * @param ids - the buckets within the map the object should be registered to
   * @param buckets - the Map to add the object to
   */
  static registerObject<U>(
    obj: U,
    ids: Set<number>,
    buckets: Map<number, Set<U>>
  ): void
  /**
   * How you get your obj into the map
   * @param obj - what you want to register in the spatial hashmap
   * @param geo - a description of the objects position and bounding box
   */
  registerObject(obj: T, geo: Geometry): void
  /**
   * Returns all the possible buckets an object's geometry is in
   * @returns - a Set of bucket ids for the passed in geometry
   */
  static getIdsForGeometry(
    pos: PointLike,
    min: PointLike,
    max: PointLike,
    cellSize: number,
    numCols: number,
    numBuckets: number
  ): Set<number>
  /**
   * Returns all the possible buckets an object's geometry is in
   * @param - geo The geometry
   * @returns - a Set of bucket ids for the passed in geometry
   */
  getIdsForGeometry(geo: Geometry): Set<number>
  /**
   * Given a Point return its bucket ID
   * @param cellSizeInv - 1 / cellsize
   * @param numCols the number of cols in the spatial map
   */
  static idForPoint(
    point: PointLike,
    cellSizeInv: number,
    numCols: number
  ): number
  /**
   * Given a Point return its bucket ID
   */
  idForPoint(point: PointLike): number
  /**
   * Given a geometry return the objects nearby
   */
  static getNearby<U>(ids: Set<number>, buckets: Map<number, Set<U>>): Set<U>
  /**
   * Given a geometry return the objects nearby
   */
  getNearby(geo: Geometry): Set<T>
}
